#!/usr/bin/env python3
""" MariaDB connector program """

import mariadb
import sys

# // Connect to MariaDB Platform //
def db_connect(**kwargs):
    """MariaDB Connection"""

    try:
        conn = mariadb.connect(
            user=kwargs["user"],
            password=kwargs["password"],
            host=kwargs["host"],
            port=kwargs["port"],
            database=kwargs["database"],
        )
        print(f"Connected to the {kwargs['database']} Database", file=sys.stderr)
    except mariadb.Error as e:
        print(f"Error connecting to MariaDB Platform: {e}", file=sys.stderr)
        sys.exit(1)
    return (conn, conn.cursor())


# // SELECT Query to MariaDB Platform //
def db_select(cur, table):
    """MariaDB SELECT Query"""

    def db_select_query(cur, query):
        """Execute the SQL SELECT Query"""

        try:
            cur.execute(query)
            return cur.fetchall()
        except:
            print("Error: Failed to get return to SELECT query", file=sys.stderr)
            sys.exit(1)

    print(f"Successfully read from the {table} Table", file=sys.stderr)
    list_ = list()
    col_query = f"SHOW COLUMNS FROM {table};"
    list_ = [tuple([x[0] for x in db_select_query(cur, col_query)])]
    main_query = f"SELECT * FROM {table};"
    list_.extend(db_select_query(cur, main_query))

    return list_


# // INSERT Query to MariaDB Platform //
def db_insert(conn, cur, query):
    """MariaDB INSERT Query"""
    print(f"query: {query}", file=sys.stderr)
    insert_query = f"INSERT INTO {query[0]} VALUES {query[1]};"
    try:
        cur.execute(insert_query)
        conn.commit()
        print(f"Successfully inserted into the {query[0]} Table", file=sys.stderr)
        return (True, insert_query)
    except:
        print(f"Error: Could not add data to the {query[0]} Table", file=sys.stderr)
        return (False, insert_query)


# // UPDATE Query to MariaDB Platform //
def db_update(conn, cur, query):
    """MariaDB UPDATE Query"""

    update_query = f"UPDATE {query[0]} SET {query[1][0]}={query[1][1]};"
    print(f"update_query: {update_query}", file=sys.stderr)
    try:
        cur.execute(update_query)
        conn.commit()
        print(f"Successfully updated the {query[0]} Table", file=sys.stderr)
        return (True, update_query)
    except:
        print(f"Error: Could not update data in the {query[0]} Table", file=sys.stderr)
        return (False, update_query)


# // DELETE Query to MariaDB Platform //
def db_delete(conn, cur, query):
    """MariaDB DELETE Query"""

    delete_query = f"DELETE FROM {query[0]} WHERE {query[1][0]}={query[1][1]};"

    try:
        cur.execute(delete_query)
        conn.commit()
        print(f"Successfully deleted from the {query[0]} Table", file=sys.stderr)
        return True
    except:
        print(
            f"Error: Could not delete data from the {query[0]} Table", file=sys.stderr
        )
        sys.exit(1)

    return True


if __name__ == "__main__":
    """Test sequence if program is ran on its own"""

    from pprint import pprint

    __author__ = "Diarmuid O'Briain"
    __copyright__ = "Copyright 2022, Technological University of the Shannon"
    __licence__ = "European Union Public Licence v1.2"
    __version__ = "2.1"
    __program__ = sys.argv[0][2:] if (sys.argv[0][:2] == "./") else sys.argv[0]

    conn, cur = db_connect(
        user="enguser", password="newengpass", host="127.0.0.1", port=3306, database="Eng"
    )
    while True:
        s = " " * 6
        test = input(f"1. SELECT{s}2. INSERT{s}3. DELETE{s}q. QUIT{s}Test What? : ")

        if test.lower() in ("1", "2", "3", "q"):
            if test == "1":
                query = ("EngProject",)
                print()
                pprint(db_select(cur, query[0]))
                print()
                continue
            elif test == "2":
                query = (
                    "EngProject",
                    (3, "ddiddly", "Diddly", "Dee", "dee@diddly.com", "Programmer"),
                )
                db_insert(conn, cur, query)
                print()
                pprint(db_select(cur, query[0]))
                print()
                continue
            elif test == "3":
                query = ("EngProject", ("Student_no", 3))
                db_delete(conn, cur, query)
                print()
                pprint(db_select(cur, query[0]))
                print()
                continue
            elif test.lower() == "q":
                print("\nTesting completed!!\n")
                sys.exit(0)
        else:
            print("Make a proper input, 1, 2 or 3!! \n")

    # // End //
    sys.exit(1)
